package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.github.pagehelper.PageInfo;
import com.github.pagehelper.page.PageMethod;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.base.service.IBasePowerLineCloudService;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadInput;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadViewOutput;
import pps.core.auth.HttpRequestClient;
import pps.core.prediction.constant.ThirdPartyApiConstant;
import pps.core.prediction.entity.WellheadDailyProductionSituationEnt;
import pps.core.prediction.mapper.WellheadDailyProductionSituationMapper;
import pps.core.prediction.service.data.wellhead_daily_production_situation.*;
import pps.core.prediction.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.XStartup;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 井口生产情况模块
 *
 * @author ZWT
 * @date 2023/09/27
 */
@XText("井口生产情况模块")
@XService
public class WellheadDailyProductionSituationService {

    /**
     * 井口生产情况--分页列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QueryWellheadDailyProductionSituationOutput}>
     */
    @XApiAnonymous
    @XText("井口生产情况--分页列表")
    @XApiPost
    public XPageResult<QueryWellheadDailyProductionSituationOutput> queryWellheadDailyProductionSituationView(XContext context, QueryWellheadDailyProductionSituationInput input) {
        WellheadDailyProductionSituationMapper mapper = context.getBean(WellheadDailyProductionSituationMapper.class);
        PageMethod.startPage(input.getPage(), input.getLimit());
        LambdaQueryWrapper<WellheadDailyProductionSituationEnt> queryWrapper = new LambdaQueryWrapper<WellheadDailyProductionSituationEnt>()
                .eq(WellheadDailyProductionSituationEnt::getWellheadId, input.getWellheadId())
                .eq(WellheadDailyProductionSituationEnt::getWellNumber, input.getWellNumber())
                .orderByDesc(WellheadDailyProductionSituationEnt::getDataDate);
        String executionCycle = input.getExecutionCycle();
        if (StringUtils.isNotEmpty(executionCycle)) {
            String[] split = StringUtils.split(executionCycle, "-");
            queryWrapper.ge(WellheadDailyProductionSituationEnt::getDataDate, DateUtil.parse(split[0], "yyyy年MM月dd日"))
                    .le(WellheadDailyProductionSituationEnt::getDataDate, DateUtil.parse(split[1], "yyyy年MM月dd日"));
        }
        List<WellheadDailyProductionSituationEnt> list = mapper.selectList(queryWrapper);
        PageInfo<WellheadDailyProductionSituationEnt> pageInfo = new PageInfo<>(list);
        List<QueryWellheadDailyProductionSituationOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryWellheadDailyProductionSituationOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /**
     * 甘特图
     * POST /prediction/wellhead-daily-production-situation/query-rpt-system-start-stop
     * 接口ID：156546795
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-156546795
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link GetWellheadDailyProductionSituationOutput}>
     */
    @XApiAnonymous
    @XText("井口生产情况--甘特图")
    @XApiPost
    public XListResult<GetWellheadDailyProductionSituationOutput> queryRptSystemStartStop(XContext context, GetWellheadDailyProductionSituationInput input) {
        String token = ServiceUtil.getToken(context);
        HttpRequestClient client = new HttpRequestClient(token);
        Map<String, Object> param = new HashMap<>(4);
        param.put("Org_Name", input.getStationName());
        param.put("WELL_NAME", input.getWellNumber());
        String url = XStartup.getCurrent().getProperty("third-party.chang-qing.url");
        url += ThirdPartyApiConstant.CQ_RPT_SYSTEM_START_STOP;
        String resultStr = client.doPostForm(url, param);
        JSONObject jsonObject = JSON.parseObject(resultStr);
        String data = jsonObject.getString("data");
        List<GetWellheadDailyProductionSituationOutput> outputs = JSON.parseArray(data, GetWellheadDailyProductionSituationOutput.class);
        if (CollUtil.isEmpty(outputs)) {
            outputs = new ArrayList<>(0);
        }
        return XListResult.success(outputs);
    }

    /**
     * 油井状态
     * POST /prediction/wellhead-daily-production-situation/query-well-real-pv-data
     * 接口ID：156558765
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-156558765
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link GetWellRealPvOutput}>
     */
    @XApiAnonymous
    @XText("井口生产情况--油井状态")
    @XApiPost
    public XListResult<GetWellRealPvOutput> queryWellRealPvData(XContext context, GetWellRealPvInput input) {
        IBasePowerLineCloudService service = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineWellheadViewOutput> result = service.getPowerLineWellheadList(context, DynamicQueryBasePowerLineWellheadInput.builder()
                .lineId(input.getLineId())
                .build()
        );
        result.throwIfFail();
        List<GetWellRealPvOutput> outputs;
        List<DynamicQueryBasePowerLineWellheadViewOutput> list = result.getResult();
        if (CollUtil.isEmpty(list)) {
            return XListResult.success(new ArrayList<>(0));
        }
        String token = ServiceUtil.getToken(context);
        HttpRequestClient client = new HttpRequestClient(token);
        String url = XStartup.getCurrent().getProperty("third-party.chang-qing.url");
        url += ThirdPartyApiConstant.CQ_WELL_REAL_PV_DATA;
        String resultStr = client.doPostForm(url, new HashMap<>(0));
        JSONObject jsonObject = JSON.parseObject(resultStr);
        String data = jsonObject.getString("data");
        outputs = JSON.parseArray(data, GetWellRealPvOutput.class);
        if (CollUtil.isEmpty(outputs)) {
            return XListResult.success(new ArrayList<>(0));
        } else {
            Set<String> collect = list.stream()
                    .map(DynamicQueryBasePowerLineWellheadViewOutput::getWellNumber)
                    .collect(Collectors.toSet());
            outputs = outputs.stream()
                    .filter(o -> collect.contains(o.getWellNumber()))
                    .collect(Collectors.toList());
        }
        return XListResult.success(outputs);
    }


    @XApiAnonymous
    @XText("井口生产情况--test")
    @XApiGet
    public XServiceResult test(XContext context) {
        context.getLogger().info("---------------------------cqToken---------------------------");
        String cqToken = "";
        String api = "";
        try {
            cqToken = ServiceUtil.getToken(context);
            context.getLogger().info("cqToken: " + cqToken);
        } catch (Exception e) {
            e.printStackTrace();
        }
        String url = XStartup.getCurrent().getProperty("third-party.chang-qing.url");
        Map<String, String> header = new HashMap<>(4);
        header.put("token", cqToken);
        context.getLogger().info("---------------------------CQ_WELL_TECH_DAILY---------------------------");
        HttpRequestClient client = new HttpRequestClient(cqToken);
        try {
            api = url + ThirdPartyApiConstant.CQ_WELL_TECH_DAILY;
            Map<String, Object> param = new HashMap<>(4);
            param.put("Calc_Date", "2024-03-15");
            String s = client.doPostForm(api, param);
            context.getLogger().info("CQ_WELL_TECH_DAILY: " + s);
        } catch (Exception e) {
            e.printStackTrace();
        }
        context.getLogger().info("---------------------------CQ_WELL_STATION_PV_DATA---------------------------");
        try {
            api = url + ThirdPartyApiConstant.CQ_WELL_STATION_PV_DATA;
            String s = client.doPostForm(api, new HashMap<>(4));
            context.getLogger().info("CQ_WELL_STATION_PV_DATA: " + s);
        } catch (Exception e) {
            e.printStackTrace();
        }
        context.getLogger().info("---------------------------CQ_RPT_SYSTEM_START_STOP---------------------------");
        try {
            api = url + ThirdPartyApiConstant.CQ_RPT_SYSTEM_START_STOP;
            Map<String, Object> param = new HashMap<>(4);
            param.put("Org_Name", "第一采油厂");
            param.put("WELL_NAME", "丹161");
            String s = client.doPostForm(api, param);
            context.getLogger().info("CQ_RPT_SYSTEM_START_STOP: " + s);
        } catch (Exception e) {
            e.printStackTrace();
        }
        context.getLogger().info("---------------------------CQ_WELL_REAL_PV_DATA---------------------------");
        try {
            api = url + ThirdPartyApiConstant.CQ_WELL_REAL_PV_DATA;
            String s = client.doPostForm(api, new HashMap<>(4));
            context.getLogger().info("CQ_WELL_REAL_PV_DATA: " + s);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return XServiceResult.OK;
    }
}